@@ -1,19 +1,28 @@
 from __future__ import annotations
-import sys
-from typing import List, Optional, Type, Any, Tuple, TYPE_CHECKING
 
-from pyglet.customtypes import AnchorX, AnchorY
+import sys
+from typing import TYPE_CHECKING, Any, ClassVar
+
 from pyglet.event import EventDispatcher
 from pyglet.font.base import grapheme_break
 from pyglet.text import runlist
-from pyglet.text.document import AbstractDocument
-from pyglet.text.layout.base import _is_pyglet_doc_run, _Line, _LayoutContext, _InlineElementBox, _InvalidRange, \
-    TextLayout
-from pyglet.text.layout.scrolling import ScrollableTextLayoutGroup, ScrollableTextDecorationGroup
+from pyglet.text.layout.base import (
+    TextLayout,
+    _AbstractBox,
+    _InlineElementBox,
+    _InvalidRange,
+    _is_pyglet_doc_run,
+    _LayoutContext,
+    _Line,
+)
+from pyglet.text.layout.scrolling import ScrollableTextDecorationGroup, ScrollableTextLayoutGroup
 
 if TYPE_CHECKING:
+    from pyglet.customtypes import AnchorX, AnchorY
     from pyglet.graphics import Batch, Group
     from pyglet.graphics.shader import ShaderProgram
+    from pyglet.graphics.vertexdomain import VertexList
+    from pyglet.text.document import AbstractDocument
 
 
 class _IncrementalLayoutContext(_LayoutContext):
@@ -23,30 +32,29 @@ class _IncrementalLayoutContext(_LayoutContext):
 
     line = None
 
-    def add_list(self, vertex_list):
+    def add_list(self, vertex_list: VertexList) -> None:
         self.line.vertex_lists.append(vertex_list)
 
-    def add_box(self, box):
+    def add_box(self, box: _AbstractBox) -> None:
         pass
 
 
-class IncrementalTextLayoutGroup(ScrollableTextLayoutGroup):
+class IncrementalTextLayoutGroup(ScrollableTextLayoutGroup):  # noqa: D101
     # Subclass so that the scissor_area isn't shared with the
     # ScrollableTextLayout. We use a class variable here so
     # that it can be set before the document glyphs are created.
-    scissor_area = 0, 0, 0, 0
+    scissor_area: ClassVar[tuple[int, int, int, int]] = 0, 0, 0, 0
 
 
-class IncrementalTextDecorationGroup(ScrollableTextDecorationGroup):
+class IncrementalTextDecorationGroup(ScrollableTextDecorationGroup):  # noqa: D101
     # Subclass so that the scissor_area isn't shared with the
     # ScrollableTextDecorationGroup. We use a class variable here so
     # that it can be set before the document glyphs are created.
-    scissor_area = 0, 0, 0, 0
+    scissor_area: ClassVar[tuple[int, int, int, int]] = 0, 0, 0, 0
 
 
 class IncrementalTextLayout(TextLayout, EventDispatcher):
-    """Displayed text suitable for interactive editing and/or scrolling
-    large documents.
+    """Displayed text suitable for interactive editing and/or scrolling large documents.
 
     Unlike :py:func:`~pyglet.text.layout.TextLayout` and
     :py:class:`~pyglet.text.layout.ScrollableTextLayout`, this class generates
@@ -55,62 +63,71 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
     video memory usage to a minimum and improve rendering speed.
 
     Changes to the document are quickly reflected in this layout, as only the
-    affected line(s) are reflowed.  Use `begin_update` and `end_update` to
+    affected line(s) are reflowed.  Use :py:meth:`~pyglet.text.layout.TextLayout.begin_update` and
+    :py:meth:`~pyglet.text.layout.TextLayout.end_update` to
     further reduce the amount of processing required.
 
     The layout can also display a text selection (text with a different
     background color).  The :py:class:`~pyglet.text.caret.Caret` class implements
     a visible text cursor and provides event handlers for scrolling, selecting and
     editing text in an incremental text layout.
+
+    Attributes:
+        group_class: Default group used to set the state for all glyphs.
+        decoration_class: Default group used to set the state for all decorations including background colors and
+            underlines.
     """
 
-    glyphs: List[Any]
-    lines: List[_Line]
+    glyphs: list[Any]
+    lines: list[_Line]
 
     _selection_start: int = 0
     _selection_end: int = 0
-    _selection_color: Tuple[int, int, int, int] = (255, 255, 255, 255)
-    _selection_background_color: Tuple[int, int, int, int] = (46, 106, 197, 255)
+    _selection_color: tuple[int, int, int, int] = (255, 255, 255, 255)
+    _selection_background_color: tuple[int, int, int, int] = (46, 106, 197, 255)
 
-    group_class: Type[IncrementalTextLayoutGroup] = IncrementalTextLayoutGroup
-    decoration_class: Type[IncrementalTextDecorationGroup] = IncrementalTextDecorationGroup
+    group_class: ClassVar[type[IncrementalTextLayoutGroup]] = IncrementalTextLayoutGroup
+    decoration_class: ClassVar[type[IncrementalTextDecorationGroup]] = IncrementalTextDecorationGroup
 
     _translate_x: int = 0
     _translate_y: int = 0
 
-    invalid_glyphs: _InvalidRange
-    invalid_flow: _InvalidRange
-    invalid_lines: _InvalidRange
-    invalid_style: _InvalidRange
-    invalid_vertex_lines: _InvalidRange
-    visible_lines: _InvalidRange
+    _invalid_glyphs: _InvalidRange
+    _invalid_flow: _InvalidRange
+    _invalid_lines: _InvalidRange
+    _invalid_style: _InvalidRange
+    _invalid_vertex_lines: _InvalidRange
+    _visible_lines: _InvalidRange
 
-    owner_runs: runlist.RunList
+    _owner_runs: runlist.RunList
 
     _width: int
     _height: int
 
-    def __init__(self, document: AbstractDocument, width: int, height: int, x: float = 0, y: float = 0, z: float = 0,
-                 anchor_x: AnchorX = 'left', anchor_y: AnchorY = 'bottom', rotation: float = 0, multiline: bool = False,
-                 dpi: Optional[float] = None, batch: Optional[Batch] = None, group: Optional[Group] = None,
-                 program: Optional[ShaderProgram] = None, wrap_lines: bool = True) -> None:
+    def __init__(self, document: AbstractDocument, width: int, height: int, x: float = 0, y: float = 0, z: float = 0,  # noqa: D107
+                 anchor_x: AnchorX = "left", anchor_y: AnchorY = "bottom", rotation: float = 0, multiline: bool = False,
+                 dpi: int | None = None, batch: Batch | None = None, group: Group | None = None,
+                 program: ShaderProgram | None = None, wrap_lines: bool = True) -> None:
 
         if width is None or height is None:
-            raise Exception("Invalid size. IncrementalTextLayout width or height cannot be None.")
+            msg = "Invalid size. IncrementalTextLayout width or height cannot be None."
+            raise Exception(msg)
 
+        #: :meta private:
         self.glyphs = []
 
-        # All lines, including hidden.
+        #: :meta private:
+        # All lines in the document, including those hidden from view.
         self.lines = []
 
-        self.invalid_glyphs = _InvalidRange()
-        self.invalid_flow = _InvalidRange()
-        self.invalid_lines = _InvalidRange()
-        self.invalid_style = _InvalidRange()
-        self.invalid_vertex_lines = _InvalidRange()
-        self.visible_lines = _InvalidRange()
+        self._invalid_glyphs = _InvalidRange()
+        self._invalid_flow = _InvalidRange()
+        self._invalid_lines = _InvalidRange()
+        self._invalid_style = _InvalidRange()
+        self._invalid_vertex_lines = _InvalidRange()
+        self._visible_lines = _InvalidRange()
 
-        self.owner_runs = runlist.RunList(0, None)
+        self._owner_runs = runlist.RunList(0, None)
 
         super().__init__(document, width, height, x, y, z, anchor_x, anchor_y, rotation, multiline, dpi, batch, group,
                          program, wrap_lines)
@@ -124,13 +141,13 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         self.foreground_decoration_group.scissor_area = area
 
     def _init_document(self) -> None:
-        assert self._document, 'Cannot remove document from IncrementalTextLayout'
+        assert self._document, "Cannot remove document from IncrementalTextLayout"
         self.on_insert_text(0, self._document.text)
 
     def _uninit_document(self) -> None:
         self.on_delete_text(0, len(self._document.text))
 
-    def _get_lines(self) -> List[_Line]:
+    def _get_lines(self) -> list[_Line]:
         return self.lines
 
     def delete(self) -> None:
@@ -145,19 +162,19 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         len_text = len(text)
         self.glyphs[start:start] = [None] * len_text
 
-        self.invalid_glyphs.insert(start, len_text)
+        self._invalid_glyphs.insert(start, len_text)
 
         # When inserting text normally with content_valign top, the text only affects the line its on and after it.
         # With other alignments, such as bottom, by adding text you may be pushing the lines above upwards.
         # To account for this, we need to invalidate the text above as well.
         if self._multiline and self._content_valign != "top":
-            visible_line = self.lines[self.visible_lines.start]
-            self.invalid_flow.invalidate(visible_line.start, start + len_text)
+            visible_line = self.lines[self._visible_lines.start]
+            self._invalid_flow.invalidate(visible_line.start, start + len_text)
 
-        self.invalid_flow.insert(start, len_text)
-        self.invalid_style.insert(start, len_text)
+        self._invalid_flow.insert(start, len_text)
+        self._invalid_style.insert(start, len_text)
 
-        self.owner_runs.insert(start, len_text)
+        self._owner_runs.insert(start, len_text)
 
         for line in self.lines:
             if line.start >= start:
@@ -170,14 +187,14 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
 
         # Same requirement as on_insert_text
         if self._multiline and self._content_valign != "top":
-            visible_line = self.lines[self.visible_lines.start]
-            self.invalid_flow.invalidate(visible_line.start, end)
+            visible_line = self.lines[self._visible_lines.start]
+            self._invalid_flow.invalidate(visible_line.start, end)
 
-        self.invalid_glyphs.delete(start, end)
-        self.invalid_flow.delete(start, end)
-        self.invalid_style.delete(start, end)
+        self._invalid_glyphs.delete(start, end)
+        self._invalid_flow.delete(start, end)
+        self._invalid_style.delete(start, end)
 
-        self.owner_runs.delete(start, end)
+        self._owner_runs.delete(start, end)
 
         size = end - start
         for line in self.lines:
@@ -185,19 +202,19 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
                 line.start = max(line.start - size, start)
 
         if start == 0:
-            self.invalid_flow.invalidate(0, 1)
+            self._invalid_flow.invalidate(0, 1)
         else:
-            self.invalid_flow.invalidate(start - 1, start)
+            self._invalid_flow.invalidate(start - 1, start)
 
         self._update()
 
     def on_style_text(self, start: int, end: int, attributes: dict[str, Any]) -> None:
-        if 'font_name' in attributes or 'font_size' in attributes or 'bold' in attributes or 'italic' in attributes:
-            self.invalid_glyphs.invalidate(start, end)
-        elif 'color' in attributes or 'background_color' in attributes:
-            self.invalid_style.invalidate(start, end)
+        if "font_name" in attributes or "font_size" in attributes or "bold" in attributes or "italic" in attributes:
+            self._invalid_glyphs.invalidate(start, end)
+        elif "color" in attributes or "background_color" in attributes:
+            self._invalid_style.invalidate(start, end)
         else:  # Attributes that change flow
-            self.invalid_flow.invalidate(start, end)
+            self._invalid_flow.invalidate(start, end)
 
         self._update()
 
@@ -205,9 +222,9 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         if not self._update_enabled:
             return
 
-        trigger_update_event = (self.invalid_glyphs.is_invalid() or
-                                self.invalid_flow.is_invalid() or
-                                self.invalid_lines.is_invalid())
+        trigger_update_event = (self._invalid_glyphs.is_invalid() or
+                                self._invalid_flow.is_invalid() or
+                                self._invalid_lines.is_invalid())
 
         len_groups = len(self.group_cache)
         # Special care if there is no text:
@@ -220,7 +237,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             self.lines[0].ascent = font.ascent
             self.lines[0].descent = font.descent
             self.lines[0].paragraph_begin = self.lines[0].paragraph_end = True
-            self.invalid_lines.invalidate(0, 1)
+            self._invalid_lines.invalidate(0, 1)
 
         self._update_glyphs()
         self._update_flow_glyphs()
@@ -238,10 +255,10 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             self._update_scissor_area()
 
         if trigger_update_event:
-            self.dispatch_event('on_layout_update')
+            self.dispatch_event("on_layout_update")
 
     def _update_glyphs(self) -> None:
-        invalid_start, invalid_end = self.invalid_glyphs.validate()
+        invalid_start, invalid_end = self._invalid_glyphs.validate()
 
         if invalid_end - invalid_start <= 0:
             return
@@ -271,13 +288,14 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
                 self.glyphs[start:end] = font.get_glyphs(text)
 
         # Update owner runs
-        self._get_owner_runs(self.owner_runs, self.glyphs, invalid_start, invalid_end)
+        owner_runs = self._get_owner_runs(self.glyphs[invalid_start:invalid_end])
+        self._owner_runs.set_runs(invalid_start, invalid_end, owner_runs)
 
         # Updated glyphs need flowing
-        self.invalid_flow.invalidate(invalid_start, invalid_end)
+        self._invalid_flow.invalidate(invalid_start, invalid_end)
 
     def _update_flow_glyphs(self) -> None:
-        invalid_start, invalid_end = self.invalid_flow.validate()
+        invalid_start, invalid_end = self._invalid_flow.validate()
 
         if invalid_end - invalid_start <= 0:
             return
@@ -303,18 +321,18 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             invalid_start = min(invalid_start, line.start)
             line.delete(self)
             self.lines[line_index] = _Line(invalid_start)
-            self.invalid_lines.invalidate(line_index, line_index + 1)
+            self._invalid_lines.invalidate(line_index, line_index + 1)
         except IndexError:
             line_index = 0
             invalid_start = 0
             line = _Line(0)
             self.lines.append(line)
-            self.invalid_lines.insert(0, 1)
+            self._invalid_lines.insert(0, 1)
 
         content_width_invalid = False
         next_start = invalid_start
 
-        for line in self._flow_glyphs(self.glyphs, self.owner_runs, invalid_start, len(self._document.text)):
+        for line in self._flow_glyphs(self.glyphs, self._owner_runs, invalid_start, len(self._document.text)):
             try:
                 old_line = self.lines[line_index]
                 old_line.delete(self)
@@ -323,10 +341,10 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
                 if old_line_width == self._content_width and new_line_width < old_line_width:
                     content_width_invalid = True
                 self.lines[line_index] = line
-                self.invalid_lines.invalidate(line_index, line_index + 1)
+                self._invalid_lines.invalidate(line_index, line_index + 1)
             except IndexError:
                 self.lines.append(line)
-                self.invalid_lines.insert(line_index, 1)
+                self._invalid_lines.insert(line_index, 1)
 
             next_start = line.start + line.length
             line_index += 1
@@ -357,14 +375,14 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             self._content_width = content_width
 
     def _update_flow_lines(self) -> None:
-        invalid_start, invalid_end = self.invalid_lines.validate()
+        invalid_start, invalid_end = self._invalid_lines.validate()
         if invalid_end - invalid_start <= 0:
             return
 
         invalid_end = self._flow_lines(self.lines, invalid_start, invalid_end)
 
         # Invalidate lines that need new vertex lists.
-        self.invalid_vertex_lines.invalidate(invalid_start, invalid_end)
+        self._invalid_vertex_lines.invalidate(invalid_start, invalid_end)
 
     def _update_visible_lines(self) -> None:
         start = sys.maxsize
@@ -377,31 +395,31 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
                 end = max(end, i) + 1
 
         # Delete newly invisible lines
-        for i in range(self.visible_lines.start, min(start, len(self.lines))):
+        for i in range(self._visible_lines.start, min(start, len(self.lines))):
             self.lines[i].delete(self)
-        for i in range(end, min(self.visible_lines.end, len(self.lines))):
+        for i in range(end, min(self._visible_lines.end, len(self.lines))):
             self.lines[i].delete(self)
 
         # Invalidate newly visible lines
-        self.invalid_vertex_lines.invalidate(start, self.visible_lines.start)
-        self.invalid_vertex_lines.invalidate(self.visible_lines.end, end)
+        self._invalid_vertex_lines.invalidate(start, self._visible_lines.start)
+        self._invalid_vertex_lines.invalidate(self._visible_lines.end, end)
 
-        self.visible_lines.start = start
-        self.visible_lines.end = end
+        self._visible_lines.start = start
+        self._visible_lines.end = end
 
-    def _update_vertex_lists(self, update_view_translation=True) -> None:
+    def _update_vertex_lists(self, update_view_translation:bool =True) -> None:
         # Find lines that have been affected by style changes
-        style_invalid_start, style_invalid_end = self.invalid_style.validate()
-        self.invalid_vertex_lines.invalidate(
+        style_invalid_start, style_invalid_end = self._invalid_style.validate()
+        self._invalid_vertex_lines.invalidate(
             self.get_line_from_position(style_invalid_start),
             self.get_line_from_position(style_invalid_end) + 1)
 
-        invalid_start, invalid_end = self.invalid_vertex_lines.validate()
+        invalid_start, invalid_end = self._invalid_vertex_lines.validate()
         if invalid_end - invalid_start <= 0:
             return
 
-        colors_iter = self.document.get_style_runs('color')
-        background_iter = self.document.get_style_runs('background_color')
+        colors_iter = self.document.get_style_runs("color")
+        background_iter = self.document.get_style_runs("background_color")
         if self._selection_end - self._selection_start > 0:
             colors_iter = runlist.OverriddenRunIterator(
                 colors_iter,
@@ -433,7 +451,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             # Early out if not visible
             if y + line.descent > self._translate_y:
                 continue
-            elif y + line.ascent < self._translate_y - self.height:
+            if y + line.ascent < self._translate_y - self.height:
                 break
 
             self._create_vertex_lists(line.x, y, self._anchor_left, top_anchor, line.start, line.boxes, context)
@@ -470,11 +488,11 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         self._update_scissor_area()
 
     @property
-    def position(self) -> Tuple[float, float, float]:
+    def position(self) -> tuple[float, float, float]:
         return self._x, self._y, self._z
 
     @position.setter
-    def position(self, position: Tuple[float, float, float]) -> None:
+    def position(self, position: tuple[float, float, float]) -> None:
         super()._set_position(position)
         self._update_view_translation()
         self._update_scissor_area()
@@ -511,7 +529,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         if width == self._width:
             return
         self._width = width
-        self.invalid_flow.invalidate(0, len(self.document.text))
+        self._invalid_flow.invalidate(0, len(self.document.text))
         self._update()
 
     @property
@@ -534,22 +552,22 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
 
     @multiline.setter
     def multiline(self, multiline: bool) -> None:
-        self.invalid_flow.invalidate(0, len(self.document.text))
+        self._invalid_flow.invalidate(0, len(self.document.text))
         self._multiline = multiline
         self._wrap_lines_invariant()
         self._update()
 
     def _update_view_translation(self) -> None:
         # Offset of content within viewport
-        for line in self.lines[self.visible_lines.start:self.visible_lines.end]:
+        for line in self.lines[self._visible_lines.start:self._visible_lines.end]:
             for box in line.boxes:
                 box.update_view_translation(self._translate_x, self._translate_y)
 
-        self.dispatch_event('on_translation_update')
+        self.dispatch_event("on_translation_update")
 
     def _update_translation(self) -> None:
         # Vertex lists are stored in the lines.
-        for line in self.lines[self.visible_lines.start:self.visible_lines.end]:
+        for line in self.lines[self._visible_lines.start:self._visible_lines.end]:
             for box in line.boxes:
                 box.update_translation(self._x, self._y, self._z)
 
@@ -563,7 +581,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         # the whole line, it will be split to 2 boxes: ("This is a ", "test")
         # This is to allow the second GlyphBox to be pushed onto the next line should it wrap in multiline.
         # "This is a test " will be created as one GlyphBox.
-        for line in self.lines[self.visible_lines.start:self.visible_lines.end]:
+        for line in self.lines[self._visible_lines.start:self._visible_lines.end]:
             # A line can have no vertex list if it's out of view OR is an empty row.
 
             # Accumulate the X accounting for multiple GlyphBoxes.
@@ -575,42 +593,45 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
     def _get_bottom_anchor(self) -> float:
         """Returns the anchor for the Y axis from the bottom."""
         height = self._height
-        if self._content_valign == 'top':
+        if self._content_valign == "top":
             offset = min(0, self._height)
-        elif self._content_valign == 'bottom':
+        elif self._content_valign == "bottom":
             offset = 0
-        elif self._content_valign == 'center':
+        elif self._content_valign == "center":
             offset = min(0, self._height) // 2
         else:
-            assert False, '`content_valign` must be either "top", "bottom", or "center".'
+            msg = '`content_valign` must be either "top", "bottom", or "center".'
+            raise Exception(msg)
 
-        if self._anchor_y == 'top':
+        if self._anchor_y == "top":
             return -height + offset
-        elif self._anchor_y == 'baseline':
+        if self._anchor_y == "baseline":
             return -height + self._ascent
-        elif self._anchor_y == 'bottom':
+        if self._anchor_y == "bottom":
             return 0
-        elif self._anchor_y == 'center':
+        if self._anchor_y == "center":
             if self._line_count == 1 and self._height is None:
                 # This "looks" more centered than considering all of the descent.
                 return (self._ascent // 2 - self._descent // 4) - height
-            else:
-                return offset - height // 2
-        else:
-            assert False, '`anchor_y` must be either "top", "bottom", "center", or "baseline".'
+
+            return offset - height // 2
+
+        msg = '`anchor_y` must be either "top", "bottom", "center", or "baseline".'
+        raise Exception(msg)
 
     @property
     def rotation(self) -> float:
+        """Rotation will always be 0 as incremental layouts cannot be rotated.
+
+        Raises:
+            NotImplementedError: Rotating IncrementalTextLayout's is not supported.
+        """
         return self._rotation
 
     @rotation.setter
-    def rotation(self, angle: float) -> None:
-        raise Exception("Rotating IncrementalTextLayout's is not supported.")
-
-    # def _set_rotation(self, rotation):
-    #     self._rotation = rotation
-    #     self.invalid_flow.invalidate(0, len(self.document.text))
-    #     self._update()
+    def rotation(self, angle: float) -> None:  # noqa: ARG002
+        msg = "Rotating IncrementalTextLayout's is not supported."
+        raise NotImplementedError(msg)
 
     @property
     def view_x(self) -> int:
@@ -620,8 +641,6 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         side of the layout bounds.  A positive value causes the text to "scroll"
         to the right.  Values are automatically clipped into the range
         ``[0, content_width - width]``
-
-        :type: int
         """
         return self._translate_x
 
@@ -643,8 +662,6 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         A negative value causes the text to "scroll" upwards.  Values outside of
         the range ``[height - content_height, 0]`` are automatically clipped in
         range.
-
-        :type: int
         """
         return self._translate_y
 
@@ -666,12 +683,9 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
 
         If ``start`` equals ``end`` no selection will be visible.
 
-        :Parameters:
-            `start` : int
-                Starting character position of selection.
-            `end` : int
-                End of selection, exclusive.
-
+        Args:
+            start: Starting character position of selection.
+            end: End of selection, exclusive.
         """
         start = max(0, start)
         end = min(end, len(self.document.text))
@@ -680,12 +694,12 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
 
         if end > self._selection_start and start < self._selection_end:
             # Overlapping, only invalidate difference
-            self.invalid_style.invalidate(min(start, self._selection_start), max(start, self._selection_start))
-            self.invalid_style.invalidate(min(end, self._selection_end), max(end, self._selection_end))
+            self._invalid_style.invalidate(min(start, self._selection_start), max(start, self._selection_start))
+            self._invalid_style.invalidate(min(end, self._selection_end), max(end, self._selection_end))
         else:
             # Non-overlapping, invalidate both ranges
-            self.invalid_style.invalidate(self._selection_start, self._selection_end)
-            self.invalid_style.invalidate(start, end)
+            self._invalid_style.invalidate(self._selection_start, self._selection_end)
+            self._invalid_style.invalidate(start, end)
 
         self._selection_start = start
         self._selection_end = end
@@ -696,9 +710,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
     def selection_start(self) -> int:
         """Starting position of the active selection.
 
-        :see: `set_selection`
-
-        :type: int
+        :see: py:meth:`~pyglet.text.layout.IncrementalTextLayout.set_selection`
         """
         return self._selection_start
 
@@ -710,9 +722,7 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
     def selection_end(self) -> int:
         """End position of the active selection (exclusive).
 
-        :see: `set_selection`
-
-        :type: int
+        :see: py:meth:`~pyglet.text.layout.IncrementalTextLayout.set_selection`
         """
         return self._selection_end
 
@@ -721,84 +731,63 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         self.set_selection(self._selection_start, end)
 
     @property
-    def selection_color(self) -> Tuple[int, int, int, int]:
+    def selection_color(self) -> tuple[int, int, int, int]:
         """Text color of active selection.
 
         The color is an RGBA tuple with components in range [0, 255].
-
-        :type: (int, int, int, int)
         """
         return self._selection_color
 
     @selection_color.setter
-    def selection_color(self, color: Tuple[int, int, int, int]) -> None:
+    def selection_color(self, color: tuple[int, int, int, int]) -> None:
         self._selection_color = color
-        self.invalid_style.invalidate(self._selection_start, self._selection_end)
+        self._invalid_style.invalidate(self._selection_start, self._selection_end)
 
     @property
-    def selection_background_color(self) -> Tuple[int, int, int, int]:
+    def selection_background_color(self) -> tuple[int, int, int, int]:
         """Background color of active selection.
 
         The color is an RGBA tuple with components in range [0, 255].
-
-        :type: (int, int, int, int)
         """
         return self._selection_background_color
 
     @selection_background_color.setter
-    def selection_background_color(self, background_color: Tuple[int, int, int, int]) -> None:
+    def selection_background_color(self, background_color: tuple[int, int, int, int]) -> None:
         self._selection_background_color = background_color
-        self.invalid_style.invalidate(self._selection_start, self._selection_end)
+        self._invalid_style.invalidate(self._selection_start, self._selection_end)
 
     # Coordinate translation
 
     def get_position_from_point(self, x: float, y: float) -> int:
-        """Get the closest document position to a point.
-
-        :Parameters:
-            `x` : int
-                X coordinate
-            `y` : int
-                Y coordinate
-
-        """
+        """Get the closest document position to a point."""
         line = self.get_line_from_point(x, y)
         return self.get_position_on_line(line, x)
 
-    def get_point_from_position(self, position, line=None):
-        """Get the X, Y coordinates of a position in the document.
+    def get_point_from_position(self, position: int, line_idx: int | None=None) -> tuple[float, float]:
+        """Get the X, Y coordinates of a character position in the document.
 
         The position that ends a line has an ambiguous point: it can be either
         the end of the line, or the beginning of the next line.  You may
         optionally specify a line index to disambiguate the case.
 
         The resulting Y coordinate gives the baseline of the line.
-
-        :Parameters:
-            `position` : int
-                Character position within document.
-            `line` : int
-                Line index.
-
-        :rtype: (int, int)
-        :return: (x, y)
         """
-        if line is None:
+        if line_idx is None:
             line = self.lines[0]
             for next_line in self.lines:
                 if next_line.start > position:
                     break
                 line = next_line
         else:
-            line = self.lines[line]
+            line = self.lines[line_idx]
 
         x = line.x
 
-        baseline = self._document.get_style('baseline', max(0, position - 1))
+        baseline = self._document.get_style("baseline", max(0, position - 1))
         if baseline is None:
             baseline = 0
         else:
-            baseline = self.parse_distance(baseline)
+            baseline = self._parse_distance(baseline)
 
         position -= line.start
         for box in line.boxes:
@@ -810,18 +799,8 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
 
         return x, line.y + baseline
 
-    def get_line_from_point(self, x, y):
-        """Get the closest line index to a point.
-
-        :Parameters:
-            `x` : int
-                X coordinate.
-            `y` : int
-                Y coordinate.
-
-        :rtype: int
-        """
-
+    def get_line_from_point(self, x: float, y: float) -> int:
+        """Get the closest line index to a point."""
         x -= self._translate_x
         y -= self._get_content_height() + self.bottom - self._translate_y
 
@@ -834,28 +813,13 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             line_index = len(self.lines) - 1
         return line_index
 
-    def get_point_from_line(self, line):
-        """Get the X, Y coordinates of a line index.
-
-        :Parameters:
-            `line` : int
-                Line index.
-
-        :rtype: (int, int)
-        :return: (x, y)
-        """
-        line = self.lines[line]
+    def get_point_from_line(self, line_idx: int) -> tuple[float, float]:
+        """Get the X, Y coordinates of a line index."""
+        line = self.lines[line_idx]
         return line.x + self._translate_x, line.y + self._translate_y
 
-    def get_line_from_position(self, position):
-        """Get the line index of a character position in the document.
-
-        :Parameters:
-            `position` : int
-                Document position.
-
-        :rtype: int
-        """
+    def get_line_from_position(self, position: int) -> int:
+        """Get the line index of a character position in the document."""
         line = -1
         for next_line in self.lines:
             if next_line.start > position:
@@ -863,29 +827,12 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             line += 1
         return line
 
-    def get_position_from_line(self, line):
-        """Get the first document character position of a given line index.
-
-        :Parameters:
-            `line` : int
-                Line index.
-
-        :rtype: int
-        """
-        return self.lines[line].start + self._x
+    def get_position_from_line(self, line_idx: int) -> int:
+        """Get the first document character position of a given line index."""
+        return int(self.lines[line_idx].start + self._x)
 
     def get_position_on_line(self, line_idx: int, x: float) -> int:
-        """Get the closest document position for a given line index and X
-        coordinate.
-
-        :Parameters:
-            `line` : int
-                Line index.
-            `x` : int
-                X coordinate.
-
-        :rtype: int
-        """
+        """Get the closest document position for a given line index and X coordinate."""
         line = self.lines[line_idx]
 
         x += self._translate_x
@@ -912,14 +859,15 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             offset = 0
         else:
             height = self._height
-            if self._content_valign == 'top':
+            if self._content_valign == "top":
                 offset = 0
-            elif self._content_valign == 'bottom':
+            elif self._content_valign == "bottom":
                 offset = max(0, self._height - self.content_height)
-            elif self._content_valign == 'center':
+            elif self._content_valign == "center":
                 offset = max(0, self._height - self.content_height) // 2
             else:
-                assert False, '`content_valign` must be either "top", "bottom", or "center".'
+                msg = '`content_valign` must be either "top", "bottom", or "center".'
+                raise Exception(msg)
 
         return height - offset
 
@@ -927,30 +875,18 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         """Returns the anchor for the X axis from the left."""
         width = self.width
 
-        if self._anchor_x == 'left':
+        if self._anchor_x == "left":
             return 0
-        elif self._anchor_x == 'center':
+        if self._anchor_x == "center":
             return -(width // 2)
-        elif self._anchor_x == 'right':
+        if self._anchor_x == "right":
             return -width
-        else:
-            assert False, '`anchor_x` must be either "left", "center", or "right".'
 
-    def get_line_count(self) -> int:
-        """Get the number of lines in the text layout.
-
-        :rtype: int
-        """
-        return self._line_count
+        msg = '`anchor_x` must be either "left", "center", or "right".'
+        raise Exception(msg)
 
     def ensure_line_visible(self, line_idx: int) -> None:
-        """Adjust `view_y` so that the line with the given index is visible.
-
-        :Parameters:
-            `line` : int
-                Line index.
-
-        """
+        """Adjust `view_y` so that the line with the given index is visible."""
         line = self.lines[line_idx]
         y1 = line.y + line.ascent
         y2 = line.y + line.descent
@@ -961,32 +897,24 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
         elif abs(self.view_y) > self.content_height - self.height:
             self.view_y = -self.content_height
 
-    def ensure_x_visible(self, x: int) -> None:
-        """Adjust `view_x` so that the given X coordinate is visible.
-
-        The X coordinate is given relative to the current `view_x`.
-
-        :Parameters:
-            `x` : int
-                X coordinate
+    def ensure_x_visible(self, x: float) -> None:
+        """Adjust ``view_x`` so that the given X coordinate is visible.
 
+        The X coordinate is given relative to the current ``view_x``.
         """
         x -= self.left
 
         if x <= self.view_x:
             self.view_x = x
 
-        elif x >= self.view_x + self.width:
-            self.view_x = x - self.width
-
-        elif (x >= self.view_x + self.width) and (self._content_width > self.width):
+        elif x >= self.view_x + self.width or (x >= self.view_x + self.width) and (self._content_width > self.width):
             self.view_x = x - self.width
 
         elif self.view_x + self.width > self._content_width:
             self.view_x = self._content_width
 
     if _is_pyglet_doc_run:
-        def on_layout_update(self):
+        def on_layout_update(self) -> None:
             """Some or all of the layout text was reflowed.
 
             Text reflow is caused by document edits or changes to the layout's
@@ -1000,5 +928,5 @@ class IncrementalTextLayout(TextLayout, EventDispatcher):
             """
 
 
-IncrementalTextLayout.register_event_type('on_layout_update')
-IncrementalTextLayout.register_event_type('on_translation_update')
+IncrementalTextLayout.register_event_type("on_layout_update")
+IncrementalTextLayout.register_event_type("on_translation_update")
